////////////////////////////////////////////////////////////////////// 
// merlin/AssociationAnalysis.h 
// (c) 2000-2010 Goncalo Abecasis
// 
// This file is distributed as part of the Goncalo source code package   
// and may not be redistributed in any form, without prior written    
// permission from the author. Permission is granted for you to       
// modify this file for your own personal use, but modified versions  
// must retain this copyright notice and must not be distributed.     
// 
// Permission is granted for you to use this file to compile Goncalo.    
// 
// All computer programs have bugs. Use this file at your own risk.   
// 
// Thursday August 12, 2010
// 
 
#ifndef __ASSOCIATION_ANALYSIS_H__
#define __ASSOCIATION_ANALYSIS_H__

#include "VarianceComponents.h"
#include "GenotypeInference.h"
#include "MerlinKinship.h"

class FamilyAnalysis;

class AssociationAnalysis : protected VarianceComponents
   {
   public:
      virtual ~AssociationAnalysis() {}

      void AnalyseAssociation(FamilyAnalysis & engine, bool threeModels = false);
      void AnalyseAssociation2df(FamilyAnalysis & engine);
      void PolygenicModel(Pedigree & ped, int t, NormalSet & mvn, IntArray * pheno, bool meanCenterCovariates = false);

      static void OutputRefinedModels(const String & prefix, Pedigree & ped);

      void OpenPerFamilyFile(int chromosome);
      void ClosePerFamilyFile(int chromosome);

   protected:
      virtual void SetupPDF(MerlinPDF & pdf, String & trait, bool threeModels = false);

      int  RetrievePhenotypes(FamilyAnalysis & engine, int modelIndex, String & traitLabel, IntArray * pheno, int & families);

      int  SuggestDigitsForEffectSize(double effect);
      int  CountVarianceComponents();
      void ReportCovariates();

   private:
      // This variable is static so it can be updated across chromosomes
      static RefinedQtlModel refinedAssociationModels;

   };

#endif


 
